<?php

namespace Drupal\image_creating_engine\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Drupal\file\Entity\File;
use Drupal\file\FileInterface;
use Drupal\Core\File\FileUrlGeneratorInterface;

/**
 * Service for generating images using built-in PHP capabilities.
 */
class InbuiltImageGenerator {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger service.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * The private tempstore.
   *
   * @var \Drupal\Core\TempStore\PrivateTempStore
   */
  protected $tempStore;

  /**
   * The file URL generator service.
   *
   * @var \Drupal\Core\File\FileUrlGeneratorInterface
   */
  protected $fileUrlGenerator;

  /**
   * Constructs an InbuiltImageGenerator object.
   *
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\TempStore\PrivateTempStoreFactory $temp_store_factory
   *   The temp store factory.
   * @param \Drupal\Core\File\FileUrlGeneratorInterface $file_url_generator
   *   The file URL generator service.
   */
  public function __construct(
    FileSystemInterface $file_system,
    EntityTypeManagerInterface $entity_type_manager,
    LoggerChannelFactoryInterface $logger_factory,
    ConfigFactoryInterface $config_factory,
    RendererInterface $renderer,
    PrivateTempStoreFactory $temp_store_factory,
    FileUrlGeneratorInterface $file_url_generator
  ) {
    $this->fileSystem = $file_system;
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger_factory->get('image_creating_engine');
    $this->configFactory = $config_factory;
    $this->renderer = $renderer;
    $this->tempStore = $temp_store_factory->get('image_creating_engine');
    $this->fileUrlGenerator = $file_url_generator;
  }

  /**
   * Generate an image from HTML and CSS.
   *
   * @param string $html
   *   The HTML content.
   * @param string $css
   *   The CSS content.
   * @param array $options
   *   An array of options for image generation.
   *
   * @return array|null
   *   An array containing the file URI and File entity, or NULL on failure.
   */
  public function generateImage($html, $css, array $options = []) {
    try {
      // Create a temporary file for the HTML content with error handling
      $temp_directory = $this->fileSystem->getTempDirectory();
      if (!$this->fileSystem->prepareDirectory($temp_directory, FileSystemInterface::CREATE_DIRECTORY)) {
        $this->logger->error('Cannot access or create temp directory: @dir', ['@dir' => $temp_directory]);
        return NULL;
      }
      
      $html_file = $temp_directory . '/temp_html_' . uniqid() . '.html';
      
      // Combine HTML and CSS.
      $full_html = '<!DOCTYPE html><html><head><meta charset="UTF-8">';
      $full_html .= '<style>' . $css . '</style></head><body>' . $html . '</body></html>';
      
      // Write the HTML to a temporary file with error checking
      $bytes_written = @file_put_contents($html_file, $full_html);
      if ($bytes_written === FALSE) {
        $error = error_get_last();
        $this->logger->error('Failed to write HTML to temporary file: @error', ['@error' => $error['message'] ?? 'Unknown error']);
        return NULL;
      }
      
      // Log successful file creation
      $this->logger->info('Created temporary HTML file: @path', ['@path' => $html_file]);
      
      // Verify the file exists after writing
      if (!file_exists($html_file)) {
        $this->logger->error('Temporary HTML file was not created: @path', ['@path' => $html_file]);
        return NULL;
      }
      
      // Set up options for image generation.
      $width = $options['width'] ?? 1200;
      $height = $options['height'] ?? 630;
      $format = strtolower($options['format'] ?? 'png');
      
      // Make sure format is valid.
      if (!in_array($format, ['png', 'jpg', 'jpeg'])) {
        $format = 'png';
      }
      
      // Set up the output file path.
      $output_dir = 'public://dynamic_image_generator/generated';
      $this->fileSystem->prepareDirectory($output_dir, FileSystemInterface::CREATE_DIRECTORY);
      $output_file = $output_dir . '/image_' . uniqid() . '.' . $format;
      
      // Generate the image
      $result = $this->generateImageFromHtml($html_file, $output_file, $width, $height, $format);
      
      if ($result) {
        // Clean up temporary file.
        $this->fileSystem->delete($html_file);
        
        // Create a file entity for the generated image.
        $file = File::create([
          'uri' => $output_file,
          'uid' => \Drupal::currentUser()->id(),
          'status' => FileInterface::STATUS_PERMANENT,
        ]);
        $file->save();
        
        $this->logger->info('Successfully generated image at: @uri', ['@uri' => $output_file]);
        
        // Return both the file URI and the File entity (more useful than just a URL)
        return [
          'uri' => $output_file,
          'file' => $file,
          'url' => $this->fileUrlGenerator->generateAbsoluteString($output_file)
        ];
      } else {
        $this->logger->error('Failed to generate image from HTML file');
      }
      
      return NULL;
    }
    catch (\Exception $e) {
      $this->logger->error('Error generating image: @error', ['@error' => $e->getMessage()]);
      return NULL;
    }
  }

  /**
   * Generate an image from HTML file.
   *
   * @param string $html_file
   *   Path to the HTML file.
   * @param string $output_file
   *   Path to the output image file.
   * @param int $width
   *   Image width.
   * @param int $height
   *   Image height.
   * @param string $format
   *   Image format (png, jpg).
   *
   * @return bool
   *   TRUE if successful, FALSE otherwise.
   */
  protected function generateImageFromHtml($html_file, $output_file, $width, $height, $format) {
    try {
      // Check if GD extension is available
      if (!extension_loaded('gd')) {
        $this->logger->error('GD extension is not available.');
        return FALSE;
      }
      
      // Verify the HTML file exists before trying to read it
      if (!file_exists($html_file)) {
        $this->logger->error('HTML file does not exist: @path', ['@path' => $html_file]);
        return FALSE;
      }
      
      // Check if we have any of the better renderer options available
      if (extension_loaded('imagick') && class_exists('Imagick')) {
        return $this->generateImageWithImagick($html_file, $output_file, $width, $height, $format);
      }
      
      // Fallback to simple GD rendering if we can't use Imagick
      return $this->generateImageWithGd($html_file, $output_file, $width, $height, $format);
    }
    catch (\Exception $e) {
      $this->logger->error('Error in generateImageFromHtml: @error', ['@error' => $e->getMessage()]);
      return FALSE;
    }
  }
  
  /**
   * Generate image using Imagick for better HTML/CSS rendering.
   */
  protected function generateImageWithImagick($html_file, $output_file, $width, $height, $format) {
    try {
      $real_html_file = $this->fileSystem->realpath($html_file);
      if (!$real_html_file) {
        $real_html_file = $html_file;
      }
      
      // Create Imagick instance
      $imagick = new \Imagick();
      
      // Set image size and background color
      $imagick->setResolution($width / 2, $height / 2); // Higher resolution for better quality
      
      // Read the HTML file with proper format
      $html_content = file_get_contents($real_html_file);
      
      // Check if this is a complete HTML document
      if (strpos($html_content, '<html') === false) {
        // Wrap in basic HTML if it's just a fragment
        $html_content = '<!DOCTYPE html><html><head><meta charset="UTF-8"></head><body>' . $html_content . '</body></html>';
      }
      
      // Save the modified HTML to a temporary file
      $temp_html = $this->fileSystem->getTempDirectory() . '/imagick_' . md5($html_content) . '.html';
      file_put_contents($temp_html, $html_content);
      
      // Read HTML - this is the key to rendering HTML with CSS
      $imagick->readImage('file://' . $temp_html);
      
      // Set output format
      if ($format == 'jpg' || $format == 'jpeg') {
        $imagick->setImageFormat('jpeg');
        $imagick->setCompressionQuality(90);
      } 
      else {
        $imagick->setImageFormat('png');
      }
      
      // Resize to desired dimensions
      $imagick->resizeImage($width, $height, \Imagick::FILTER_LANCZOS, 1);
      
      // Write the image to the output file
      $imagick->writeImage($output_file);
      $imagick->clear();
      
      // Clean up temporary HTML file
      if (file_exists($temp_html)) {
        unlink($temp_html);
      }
      
      return file_exists($output_file);
    }
    catch (\Exception $e) {
      $this->logger->error('Error generating image with Imagick: @error', ['@error' => $e->getMessage()]);
      // Fallback to GD if Imagick fails
      return $this->generateImageWithGd($html_file, $output_file, $width, $height, $format);
    }
  }
  
  /**
   * Generate image using GD with improved HTML/CSS parsing.
   */
  protected function generateImageWithGd($html_file, $output_file, $width, $height, $format) {
    try {
      // Get the HTML content
      $real_html_file = $this->fileSystem->realpath($html_file);
      if (!$real_html_file) {
        $real_html_file = $html_file;
      }
      
      $html_content = file_get_contents($real_html_file);
      
      // Create a blank image
      $image = imagecreatetruecolor($width, $height);
      if (!$image) {
        $this->logger->error('Failed to create image resource');
        return FALSE;
      }
      
      // Set background color to white by default
      $background_color = imagecolorallocate($image, 255, 255, 255);
      imagefill($image, 0, 0, $background_color);
      
      // Try to extract background color from HTML
      preg_match('/body\s*{[^}]*background-color:\s*([^;]+);/i', $html_content, $matches);
      if (!empty($matches[1])) {
        $bg_color = $this->parseColor($matches[1]);
        if ($bg_color) {
          $background_color = imagecolorallocate($image, $bg_color[0], $bg_color[1], $bg_color[2]);
          imagefill($image, 0, 0, $background_color);
        }
      }
      
      // Try to load and parse the HTML with DOMDocument for better structure handling
      if (class_exists('DOMDocument')) {
        $dom = new \DOMDocument();
        
        // Suppress errors during HTML parsing
        libxml_use_internal_errors(true);
        $dom->loadHTML($html_content);
        libxml_clear_errors();
        
        // Create a canvas context for drawing
        $context = [
          'x' => 20,  // starting x position
          'y' => 20,  // starting y position
          'max_width' => $width - 40, // Available width
          'line_height' => 20,
          'font_size' => 3,  // GD font size (1-5)
          'text_color' => imagecolorallocate($image, 0, 0, 0), // Default black
          'current_style' => [
            'font_size' => 3,
            'color' => [0, 0, 0],
            'weight' => 'normal',
          ],
        ];
        
        // Extract CSS colors for text
        preg_match('/body\s*{[^}]*color:\s*([^;]+);/i', $html_content, $matches);
        if (!empty($matches[1])) {
          $color = $this->parseColor($matches[1]);
          if ($color) {
            $context['text_color'] = imagecolorallocate($image, $color[0], $color[1], $color[2]);
            $context['current_style']['color'] = $color;
          }
        }
        
        // Improved rendering approach: render the DOM tree with basic layout
        $this->renderDomElement($dom->documentElement, $image, $context);
        
        // Render image elements if any
        $images = $dom->getElementsByTagName('img');
        foreach ($images as $img) {
          $src = $img->getAttribute('src');
          if (!empty($src)) {
            $this->renderImageElement($image, $src, $context);
          }
        }
      }
      else {
        // Very basic fallback if DOMDocument isn't available
        $this->renderBasicText($image, strip_tags($html_content), $width, $height);
      }
      
      // Save the image to the output file
      $result = FALSE;
      if ($format == 'jpg' || $format == 'jpeg') {
        $result = imagejpeg($image, $output_file, 90);
      }
      else {
        $result = imagepng($image, $output_file);
      }
      
      // Free up memory
      imagedestroy($image);
      
      return file_exists($output_file);
    }
    catch (\Exception $e) {
      $this->logger->error('Error generating image with GD: @error', ['@error' => $e->getMessage()]);
      return FALSE;
    }
  }
  
  /**
   * Render a DOM element and its children to the image.
   */
  protected function renderDomElement($element, $image, &$context) {
    if (!$element || $element->nodeType !== XML_ELEMENT_NODE) {
      return;
    }
    
    // Save the original context to restore after processing this element
    $original_context = $context;
    
    // Handle element specific styling
    switch ($element->nodeName) {
      case 'h1':
        $context['font_size'] = 5;
        $context['line_height'] = 30;
        $context['y'] += 10; // Add some margin
        break;
        
      case 'h2':
        $context['font_size'] = 4;
        $context['line_height'] = 25;
        $context['y'] += 8; // Add some margin
        break;
        
      case 'h3':
      case 'h4':
        $context['font_size'] = 3;
        $context['line_height'] = 22;
        $context['y'] += 5; // Add some margin
        break;
        
      case 'p':
        $context['y'] += 10; // Add paragraph margin
        break;
        
      case 'div':
        // Check for inline styles
        $style = $element->getAttribute('style');
        if (!empty($style)) {
          $this->applyInlineStyles($style, $context, $image);
        }
        
        // Check for background color in style
        if (preg_match('/background-color:\s*([^;]+);/i', $style, $matches)) {
          $bg_color = $this->parseColor($matches[1]);
          if ($bg_color) {
            // Draw a filled rectangle for div background
            $div_width = $context['max_width'];
            $div_height = $context['line_height'];
            $color = imagecolorallocate($image, $bg_color[0], $bg_color[1], $bg_color[2]);
            imagefilledrectangle($image, $context['x'], $context['y'], 
                                $context['x'] + $div_width, $context['y'] + $div_height, $color);
          }
        }
        break;
        
      case 'strong':
      case 'b':
        $context['font_size'] = min(5, $context['font_size'] + 1);
        break;
        
      case 'img':
        // Skip - we handle images separately
        break;
    }
    
    // Handle text nodes
    if ($element->nodeType === XML_TEXT_NODE || !$element->hasChildNodes()) {
      $text = $element->textContent;
      if (trim($text)) {
        $this->renderText($image, $text, $context);
      }
    }
    else {
      // Process child nodes recursively
      foreach ($element->childNodes as $child) {
        $this->renderDomElement($child, $image, $context);
      }
    }
    
    // Add appropriate spacing after block elements
    if (in_array($element->nodeName, ['div', 'p', 'h1', 'h2', 'h3', 'h4', 'ul', 'ol'])) {
      $context['y'] += 10; // Add bottom margin
      $context['x'] = $original_context['x']; // Reset x position for block elements
    }
    
    // Restore original context (except for y position)
    $context['font_size'] = $original_context['font_size'];
    $context['line_height'] = $original_context['line_height'];
    $context['text_color'] = $original_context['text_color'];
    $context['current_style'] = $original_context['current_style'];
  }
  
  /**
   * Apply inline CSS styles to the current context.
   */
  protected function applyInlineStyles($style_string, &$context, $image) {
    // Extract color
    if (preg_match('/color:\s*([^;]+);/i', $style_string, $matches)) {
      $color = $this->parseColor($matches[1]);
      if ($color) {
        $context['text_color'] = imagecolorallocate($image, $color[0], $color[1], $color[2]);
        $context['current_style']['color'] = $color;
      }
    }
    
    // Extract font size
    if (preg_match('/font-size:\s*(\d+)px;/i', $style_string, $matches)) {
      $size = (int) $matches[1];
      // Map CSS px size to GD font sizes (1-5)
      if ($size <= 10) $context['font_size'] = 1;
      else if ($size <= 14) $context['font_size'] = 2;
      else if ($size <= 18) $context['font_size'] = 3;
      else if ($size <= 24) $context['font_size'] = 4;
      else $context['font_size'] = 5;
      
      $context['current_style']['font_size'] = $context['font_size'];
    }
    
    // Extract font weight
    if (preg_match('/font-weight:\s*([^;]+);/i', $style_string, $matches)) {
      $weight = trim(strtolower($matches[1]));
      if ($weight === 'bold' || $weight >= 600) {
        $context['font_size'] = min(5, $context['font_size'] + 1);
        $context['current_style']['weight'] = 'bold';
      }
    }
  }
  
  /**
   * Render text to the image with word wrapping.
   */
  protected function renderText($image, $text, &$context) {
    $text = trim($text);
    if (empty($text)) {
      return;
    }
    
    // Word wrap text to fit within the max width
    $font_width = imagefontwidth($context['font_size']);
    $chars_per_line = floor($context['max_width'] / $font_width);
    
    $lines = explode("\n", wordwrap($text, $chars_per_line, "\n"));
    
    foreach ($lines as $line) {
      $line = trim($line);
      if (!empty($line)) {
        imagestring($image, $context['font_size'], $context['x'], $context['y'], $line, $context['text_color']);
        $context['y'] += $context['line_height'];
      }
    }
  }
  
  /**
   * Render an image element to the GD image.
   */
  protected function renderImageElement($image, $src, &$context) {
    try {
      // Only process http/https URLs or data URIs
      if (strpos($src, 'http') === 0 || strpos($src, 'data:image') === 0) {
        $img = null;
        
        // Handle URLs
        if (strpos($src, 'http') === 0) {
          $img_data = @file_get_contents($src);
          if ($img_data) {
            $img = @imagecreatefromstring($img_data);
          }
        }
        // Handle data URIs
        else if (strpos($src, 'data:image') === 0) {
          $data = explode(',', $src);
          if (count($data) > 1) {
            $img_data = base64_decode($data[1]);
            if ($img_data) {
              $img = @imagecreatefromstring($img_data);
            }
          }
        }
        
        if ($img) {
          // Get image dimensions
          $img_width = imagesx($img);
          $img_height = imagesy($img);
          
          // Scale down if needed
          $max_img_width = $context['max_width'];
          if ($img_width > $max_img_width) {
            $scale = $max_img_width / $img_width;
            $new_width = $max_img_width;
            $new_height = $img_height * $scale;
            
            // Create a scaled version
            $scaled_img = imagecreatetruecolor($new_width, $new_height);
            imagecopyresampled($scaled_img, $img, 0, 0, 0, 0, $new_width, $new_height, $img_width, $img_height);
            
            // Copy the scaled image to the main canvas
            imagecopy($image, $scaled_img, $context['x'], $context['y'], 0, 0, $new_width, $new_height);
            
            // Update y position after image
            $context['y'] += $new_height + 10;
            
            // Free memory
            imagedestroy($scaled_img);
          }
          else {
            // Copy the image directly if no scaling needed
            imagecopy($image, $img, $context['x'], $context['y'], 0, 0, $img_width, $img_height);
            
            // Update y position after image
            $context['y'] += $img_height + 10;
          }
          
          // Free memory
          imagedestroy($img);
        }
      }
    }
    catch (\Exception $e) {
      // Log but continue if image rendering fails
      $this->logger->warning('Failed to render image @src: @error', [
        '@src' => $src,
        '@error' => $e->getMessage(),
      ]);
    }
  }
  
  /**
   * Render basic text with minimal formatting (fallback method).
   */
  protected function renderBasicText($image, $text, $width, $height) {
    $text_color = imagecolorallocate($image, 0, 0, 0);
    $font_size = 3;
    $y = 20;
    
    // Word wrap text to fit image width
    $font_width = imagefontwidth($font_size);
    $chars_per_line = floor(($width - 40) / $font_width);
    
    $lines = explode("\n", wordwrap($text, $chars_per_line, "\n"));
    
    foreach ($lines as $line) {
      $line = trim($line);
      if (!empty($line)) {
        imagestring($image, $font_size, 20, $y, $line, $text_color);
        $y += 20;
        
        if ($y >= $height - 20) {
          break; // Stop if we reach the bottom of the image
        }
      }
    }
  }

  /**
   * Parse a CSS color value into RGB components.
   *
   * @param string $color
   *   CSS color value (hex, rgb, or named color).
   *
   * @return array|null
   *   Array with RGB values [r, g, b] or NULL if parsing failed.
   */
  protected function parseColor($color) {
    // Clean the color string
    $color = trim(strtolower($color));
    
    // Handle hex colors
    if (preg_match('/^#([0-9a-f]{3}|[0-9a-f]{6})$/', $color)) {
      if (strlen($color) == 4) {
        // Convert #rgb to #rrggbb
        $r = hexdec(substr($color, 1, 1) . substr($color, 1, 1));
        $g = hexdec(substr($color, 2, 1) . substr($color, 2, 1));
        $b = hexdec(substr($color, 3, 1) . substr($color, 3, 1));
        return [$r, $g, $b];
      } else {
        // Standard #rrggbb
        $r = hexdec(substr($color, 1, 2));
        $g = hexdec(substr($color, 3, 2));
        $b = hexdec(substr($color, 5, 2));
        return [$r, $g, $b];
      }
    }
    
    // Handle rgb() format
    if (preg_match('/^rgb\((\d+),\s*(\d+),\s*(\d+)\)$/', $color, $matches)) {
      return [
        min(255, max(0, (int) $matches[1])),
        min(255, max(0, (int) $matches[2])),
        min(255, max(0, (int) $matches[3])),
      ];
    }
    
    // Handle common color names
    $colors = [
      'black' => [0, 0, 0],
      'white' => [255, 255, 255],
      'red' => [255, 0, 0],
      'green' => [0, 128, 0],
      'blue' => [0, 0, 255],
      'yellow' => [255, 255, 0],
      'gray' => [128, 128, 128],
      'grey' => [128, 128, 128],
      'purple' => [128, 0, 128],
      'orange' => [255, 165, 0],
      'brown' => [165, 42, 42],
      'pink' => [255, 192, 203],
      'transparent' => [255, 255, 255], // Treat as white
    ];
    
    if (isset($colors[$color])) {
      return $colors[$color];
    }
    
    // Default to black if color can't be parsed
    return [0, 0, 0];
  }

}
