<?php

namespace Drupal\dynamic_image_generator\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\node\Entity\Node;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller for Dynamic Image Generator operations.
 */
class DynamicImageGeneratorController extends ControllerBase {

  /**
   * Generate preview with current form data (real-time values).
   */
  public function previewWithCurrentData($poster_entity, $node, Request $request) {
    $response = new AjaxResponse();
    
    try {
      // Load the poster entity properly
      if (is_numeric($poster_entity)) {
        $poster_entity = \Drupal::entityTypeManager()->getStorage('poster_entity')->load($poster_entity);
      }
      
      if (!$poster_entity) {
        throw new \Exception('Image template not found');
      }
      
      $service = \Drupal::service('dynamic_image_generator.dynamic_image_generator_service');
      
      // Get form data from request - CHECK BOTH GET AND POST
      $form_data_json = $request->query->get('form_data') ?: $request->request->get('form_data');
      $form_data = $form_data_json ? json_decode($form_data_json, TRUE) : [];
      
      // Check if this is a live preview with template changes
      $is_live_preview = isset($form_data['preview_mode']) && $form_data['preview_mode'] === 'live';
      $is_live_trigger = isset($form_data['trigger_type']) && $form_data['trigger_type'] === 'live_preview_button';
      $is_node_form_preview = isset($form_data['preview_mode']) && $form_data['preview_mode'] === 'node_form';
      $is_node_form_trigger = isset($form_data['trigger_type']) && $form_data['trigger_type'] === 'node_form_preview';
      
      // Only process live preview requests or node form preview requests
      if (!($is_live_preview && $is_live_trigger) && !($is_node_form_preview && $is_node_form_trigger)) {
        // Return empty response to prevent unwanted popups
        return $response;
      }
      
      // For node form previews, use the current node ID from form data or URL parameter
      if ($is_node_form_preview && isset($form_data['current_node_id'])) {
        $selected_node_id = $form_data['current_node_id'];
        $detection_method = 'node_form_current';
      } elseif ($is_node_form_preview && $node !== 'new' && is_numeric($node)) {
        $selected_node_id = $node;
        $detection_method = 'node_form_url';
      } else {
        // Check for selected_node in form data (live preview)
        if (isset($form_data['selected_node']) && !empty($form_data['selected_node'])) {
          $autocomplete_value = $form_data['selected_node'];
          
          // Extract node ID from autocomplete value (format: "Title (123)")
          if (preg_match('/\((\d+)\)$/', $autocomplete_value, $matches)) {
            $selected_node_id = $matches[1];
            $detection_method = 'form_data_selected_node';
          }
        }
      }
      
      // Load the node entity if we found an ID
      if ($selected_node_id) {
        $entity_id = $selected_node_id;
        $node_entity = \Drupal::entityTypeManager()->getStorage('node')->load($entity_id);
      }
      
      // Prepare custom tokens
      $custom_tokens = [];
      
      if ($node_entity) {
        // Use the actual node's data for tokens
        $custom_tokens = $this->getNodeTokens($node_entity);
      } else {
        // No specific node selected, use sample data
        $custom_tokens = [
          'title' => 'Sample Title',
          'node:title' => 'Sample Title',
          'body' => 'This is sample body content for the preview.',
          'node:body' => 'This is sample body content for the preview.',
        ];
      }
      
      // Always add these default tokens
      $custom_tokens['site_name'] = \Drupal::config('system.site')->get('name');
      $custom_tokens['site:name'] = \Drupal::config('system.site')->get('name');
      $custom_tokens['date'] = \Drupal::service('date.formatter')->format(time(), 'medium');
      
      // If this is a live preview, add custom HTML/CSS from form
      if ($is_live_preview && isset($form_data['template_html']) && isset($form_data['template_css'])) {
        $custom_tokens['_custom_html'] = $form_data['template_html'];
        $custom_tokens['_custom_css'] = $form_data['template_css'];
      }
      
      // Generate preview image
      if ($entity_id && $node_entity) {
        $image_url = $service->generatePosterImage($poster_entity->id(), $custom_tokens, $entity_id);
        $preview_type = 'with selected content (' . $node_entity->getTitle() . ')';
      } else {
        $image_url = $service->generatePreviewImage($poster_entity->id(), $custom_tokens);
        $preview_type = 'with sample data';
      }
      
      if ($is_live_preview) {
        $preview_type .= ' + current template changes';
      } elseif ($is_node_form_preview) {
        $preview_type .= ' + current form data';
      }
      
      if ($image_url) {
        $content = [
          '#theme' => 'container',
          '#attributes' => ['class' => ['dynamic-image-preview']],
          '#children' => [
            'header' => [
              '#markup' => '<div style="background: #f8f9fa; padding: 15px; margin-bottom: 20px; border-radius: 4px; border-left: 4px solid #007bff;">
                <h4 style="margin: 0 0 5px 0; color: #2c3e50;">Preview: ' . $poster_entity->label() . '</h4>
                <p style="margin: 0; color: #6c757d; font-size: 0.9em;">Generated ' . $preview_type . '. No image is saved during preview.</p>
              </div>',
            ],
            'image' => [
              '#theme' => 'image',
              '#uri' => $image_url,
              '#alt' => $this->t('Preview of @name', ['@name' => $poster_entity->label()]),
              '#attributes' => [
                'style' => 'max-width: 600px; max-height: 400px; width: auto; height: auto; border: 1px solid #ddd; border-radius: 4px; box-shadow: 0 2px 4px rgba(0,0,0,0.1); display: block; margin: 0 auto;',
                'class' => ['generated-preview-image'],
              ],
            ],
            'image_info' => [
              '#markup' => '<div style="text-align: center; margin-top: 15px; padding: 10px; background: #f0f8ff; border-radius: 4px;">
                <p style="margin: 0; color: #666; font-size: 0.85em;">
                  <strong>Full Size:</strong> <a href="' . $image_url . '" target="_blank" style="color: #007bff; text-decoration: none;">View Original Size</a> | 
                  <a href="' . $image_url . '" download style="color: #007bff; text-decoration: none;">Download Image</a>
                </p>
              </div>',
            ],
          ],
        ];
        
        $title = $entity_id ? 
          $this->t('🔴 LIVE: @name (using selected content)', ['@name' => $poster_entity->label()]) : 
          $this->t('🔴 LIVE: @name (sample data)', ['@name' => $poster_entity->label()]);
        
        // Render the content properly to HTML string
        $renderer = \Drupal::service('renderer');
        $rendered_content = $renderer->render($content);
        
      } else {
        $rendered_content = '<div class="messages messages--error" style="margin: 20px 0;">' . 
          $this->t('Failed to generate preview. Check logs for details.') . 
          '</div>';
        $title = $this->t('Preview Error');
      }
      
      // Generate the single modal response with proper content - reduced modal size
      $response->addCommand(new OpenModalDialogCommand($title, $rendered_content, [
        'width' => 700,
        'height' => 600,
        'resizable' => true,
        'draggable' => true,
      ]));
      
    } catch (\Exception $e) {
      // Handle errors for both live preview and node form preview
      if ((isset($form_data['trigger_type']) && $form_data['trigger_type'] === 'live_preview_button') ||
          (isset($form_data['trigger_type']) && $form_data['trigger_type'] === 'node_form_preview')) {
        $content = [
          '#markup' => '<div class="messages messages--error">' . 
            $this->t('Error generating preview: @error', ['@error' => $e->getMessage()]) . 
            '</div>',
        ];
        
        $response->addCommand(new OpenModalDialogCommand($this->t('Preview Error'), $content));
      }
      // For other requests, just return empty response
    }
    
    return $response;
  }

  /**
   * Extract tokens from form data.
   */
  protected function extractTokensFromFormData(array $form_data, $content_type) {
    $tokens = [];
    
    foreach ($form_data as $field_name => $value) {
      // Clean field name (remove array notation)
      $clean_name = preg_replace('/\[.*?\]/', '', $field_name);
      
      // Skip system fields
      if (in_array($clean_name, ['form_build_id', 'form_token', 'form_id', 'op', 'preview_mode'])) {
        continue;
      }
      
      // Handle HTML and CSS content from live preview
      if ($clean_name === 'template_html') {
        $tokens['_custom_html'] = $value;
        continue;
      }
      if ($clean_name === 'template_css') {
        $tokens['_custom_css'] = $value;
        continue;
      }
      
      // Convert field names to token format
      if (strpos($clean_name, 'field_') === 0 || in_array($clean_name, ['title', 'body'])) {
        $token_name = str_replace('field_', '', $clean_name);
        $tokens[$token_name] = is_array($value) ? implode(', ', $value) : $value;
        
        // Also add with node: prefix for compatibility
        $tokens['node:' . $clean_name] = $tokens[$token_name];
      }
    }
    
    return $tokens;
  }

  /**
   * Get tokens from a node entity.
   */
  protected function getNodeTokens($node) {
    $tokens = [];
    
    // Basic node info
    $tokens['title'] = $node->getTitle();
    $tokens['node:title'] = $node->getTitle();
    $tokens['nid'] = $node->id();
    $tokens['node:nid'] = $node->id();
    
    // Get body field if it exists
    if ($node->hasField('body') && !$node->get('body')->isEmpty()) {
      $body_value = $node->get('body')->first();
      if ($body_value) {
        $body_text = strip_tags($body_value->value);
        $tokens['body'] = $body_text;
        $tokens['node:body'] = $body_text;
        $tokens['node:body:value'] = $body_text;
        
        // Add summary if available
        if (!empty($body_value->summary)) {
          $tokens['body:summary'] = strip_tags($body_value->summary);
          $tokens['node:body:summary'] = strip_tags($body_value->summary);
        }
      }
    }
    
    // Get all custom field values
    $field_count = 0;
    
    // Define known base fields to skip
    $base_fields = [
      'nid', 'uuid', 'vid', 'langcode', 'type', 'revision_timestamp', 
      'revision_uid', 'revision_log', 'status', 'uid', 'title', 'created', 
      'changed', 'promote', 'sticky', 'default_langcode', 'revision_default',
      'revision_translation_affected', 'path', 'body', 'comment'
    ];
    
    foreach ($node->getFields() as $field_name => $field) {
      // Skip if field is empty
      if ($field->isEmpty()) {
        continue;
      }
      
      // Skip known base fields
      if (in_array($field_name, $base_fields)) {
        continue;
      }
      
      // Skip fields that don't start with 'field_' (likely base fields)
      if (strpos($field_name, 'field_') !== 0) {
        continue;
      }
      
      $field_value = $field->first();
      if ($field_value) {
        $clean_field_name = str_replace('field_', '', $field_name);
        $field_type = $field->getFieldDefinition()->getType();
        
        // Handle different field types
        switch ($field_type) {
          case 'string':
          case 'string_long':
          case 'text':
          case 'text_long':
          case 'text_with_summary':
            $value = is_object($field_value) && isset($field_value->value) ? strip_tags($field_value->value) : (string) $field_value;
            $tokens[$clean_field_name] = $value;
            $tokens['node:' . $field_name] = $value;
            $tokens['node:' . $field_name . ':value'] = $value;
            $field_count++;
            break;
            
          case 'integer':
          case 'decimal':
          case 'float':
            $value = is_object($field_value) && isset($field_value->value) ? $field_value->value : (string) $field_value;
            $tokens[$clean_field_name] = $value;
            $tokens['node:' . $field_name] = $value;
            $tokens['node:' . $field_name . ':value'] = $value;
            $field_count++;
            break;
            
          case 'boolean':
            $value = is_object($field_value) && isset($field_value->value) ? ($field_value->value ? 'Yes' : 'No') : 'No';
            $tokens[$clean_field_name] = $value;
            $tokens['node:' . $field_name] = $value;
            $field_count++;
            break;
            
          case 'entity_reference':
          case 'entity_reference_revisions':
            // For entity references, get the label
            if (is_object($field_value) && isset($field_value->entity) && $field_value->entity) {
              $referenced_entity = $field_value->entity;
              $label = $referenced_entity->label();
              $tokens[$clean_field_name] = $label;
              $tokens['node:' . $field_name] = $label;
              $tokens['node:' . $field_name . ':entity:label'] = $label;
              $field_count++;
              
              // For taxonomy terms, also add the ID
              if ($referenced_entity->getEntityTypeId() === 'taxonomy_term') {
                $tokens[$clean_field_name . ':tid'] = $referenced_entity->id();
                $tokens['node:' . $field_name . ':target_id'] = $referenced_entity->id();
              }
            }
            break;
            
          case 'image':
          case 'file':
            // For file/image fields, provide the URL
            if (is_object($field_value) && isset($field_value->entity) && $field_value->entity) {
              $file = $field_value->entity;
              try {
                $file_url = \Drupal::service('file_url_generator')->generateAbsoluteString($file->getFileUri());
                $tokens[$clean_field_name] = $file_url;
                $tokens['node:' . $field_name] = $file_url;
                $tokens['node:' . $field_name . ':url'] = $file_url;
                $tokens['node:' . $field_name . ':entity:url'] = $file_url;
                $field_count++;
                
                // Add alt text for images
                if ($field_type === 'image' && !empty($field_value->alt)) {
                  $tokens[$clean_field_name . ':alt'] = $field_value->alt;
                  $tokens['node:' . $field_name . ':alt'] = $field_value->alt;
                }
              } catch (\Exception $e) {
                // Log error but continue processing
              }
            }
            break;
            
          default:
            // Fallback for other field types
            if (is_object($field_value)) {
              if (isset($field_value->value)) {
                $tokens[$clean_field_name] = $field_value->value;
                $tokens['node:' . $field_name] = $field_value->value;
                $field_count++;
              } elseif (method_exists($field_value, '__toString')) {
                $tokens[$clean_field_name] = (string) $field_value;
                $tokens['node:' . $field_name] = (string) $field_value;
                $field_count++;
              }
            } else {
              $tokens[$clean_field_name] = (string) $field_value;
              $tokens['node:' . $field_name] = (string) $field_value;
              $field_count++;
            }
        }
      }
    }
    
    // Add node metadata
    $tokens['node:created'] = \Drupal::service('date.formatter')->format($node->getCreatedTime(), 'medium');
    $tokens['node:created:short'] = \Drupal::service('date.formatter')->format($node->getCreatedTime(), 'short');
    $tokens['node:created:long'] = \Drupal::service('date.formatter')->format($node->getCreatedTime(), 'long');
    $tokens['node:changed'] = \Drupal::service('date.formatter')->format($node->getChangedTime(), 'medium');
    $tokens['node:author:name'] = $node->getOwner() ? $node->getOwner()->getDisplayName() : 'Anonymous';
    $tokens['node:author:uid'] = $node->getOwnerId();
    $tokens['node:url'] = $node->toUrl('canonical', ['absolute' => TRUE])->toString();
    $tokens['node:type'] = $node->bundle();
    $tokens['node:type:name'] = $node->type->entity->label();
    
    // Add some variations for common use cases
    $tokens['author'] = $tokens['node:author:name'];
    $tokens['created'] = $tokens['node:created'];
    $tokens['type'] = $tokens['node:type:name'];
    
    return $tokens;
  }

  /**
   * Generate a preview image.
   */
  public function generatePreview($poster_entity, Request $request) {
    // ...existing code...
  }

  /**
   * Debug endpoint to check form state and node selection.
   */
  public function debugNodeSelection(Request $request) {
    // ...existing code...
  }

  /**
   * Select content for generation.
   */
  public function selectContent($poster_entity, Request $request) {
    return new JsonResponse(['status' => 'not implemented']);
  }

  /**
   * Generate image for a specific node.
   */
  public function generateForNode($poster_entity, Node $node, Request $request) {
    return new JsonResponse(['status' => 'not implemented']);
  }

  /**
   * Search content API endpoint.
   */
  public function searchContent(Request $request) {
    return new JsonResponse([]);
  }

  /**
   * Generate poster via API.
   */
  public function generatePoster($poster_entity_id, Request $request) {
    return new JsonResponse(['status' => 'not implemented']);
  }

  /**
   * Get entity title for page titles.
   */
  public function entityTitle($poster_entity = NULL) {
    if (!$poster_entity) {
      return $this->t('Dynamic Image Generator');
    }
    
    if (is_numeric($poster_entity)) {
      $poster_entity = \Drupal::entityTypeManager()->getStorage('poster_entity')->load($poster_entity);
    }
    
    if (!$poster_entity) {
      return $this->t('Dynamic Image Generator');
    }
    
    return $poster_entity->label();
  }

  /**
   * Get content type fields for AJAX callback.
   */
  public function getContentTypeFields($content_type, Request $request) {
    return new JsonResponse([]);
  }

  /**
   * Extract form data from node form submission.
   */
  protected function extractNodeFormData(array $form_data) {
    $tokens = [];
    
    foreach ($form_data as $field_name => $value) {
      // Skip system fields
      if (in_array($field_name, ['form_build_id', 'form_token', 'form_id', 'op', 'preview_mode', 'current_node_id', 'trigger_type'])) {
        continue;
      }
      
      // Handle title field
      if (strpos($field_name, 'title[') === 0 && is_array($value)) {
        $title_value = reset($value);
        if (!empty($title_value)) {
          $tokens['title'] = $title_value;
          $tokens['node:title'] = $title_value;
        }
      }
      
      // Handle body field
      if (strpos($field_name, 'body[') === 0 && is_array($value)) {
        $body_data = reset($value);
        if (is_array($body_data) && isset($body_data['value'])) {
          $tokens['body'] = strip_tags($body_data['value']);
          $tokens['node:body'] = strip_tags($body_data['value']);
          $tokens['node:body:value'] = strip_tags($body_data['value']);
        }
      }
      
      // Handle custom fields
      if (strpos($field_name, 'field_') === 0) {
        // Extract field name
        preg_match('/^(field_[^[]+)/', $field_name, $matches);
        if (isset($matches[1])) {
          $clean_field_name = $matches[1];
          $token_name = str_replace('field_', '', $clean_field_name);
          
          if (is_array($value)) {
            $field_value = reset($value);
            if (is_array($field_value)) {
              // Handle different field structures
              if (isset($field_value['value'])) {
                $tokens[$token_name] = $field_value['value'];
                $tokens['node:' . $clean_field_name] = $field_value['value'];
              } elseif (isset($field_value['target_id'])) {
                // Entity reference field - load the referenced entity
                try {
                  $referenced_entity = \Drupal::entityTypeManager()->getStorage('node')->load($field_value['target_id']);
                  if ($referenced_entity) {
                    $tokens[$token_name] = $referenced_entity->label();
                    $tokens['node:' . $clean_field_name] = $referenced_entity->label();
                  }
                } catch (\Exception $e) {
                  // Continue if entity can't be loaded
                }
              }
            } else {
              $tokens[$token_name] = $field_value;
              $tokens['node:' . $clean_field_name] = $field_value;
            }
          }
        }
      }
    }
    
    return $tokens;
  }

}